@@ -284,14 +284,20 @@ class RandomOptimizer:
         x0, bounds = problem.get_x0_w_bounds()
         last_x = np.copy(x0)
 
-        # problem.set_x(last_x)
+        last_f = np.inf
 
         try:
-            last_f = problem(last_x)
-            global_state_history.append(deepcopy(problem.global_state))
-            f_history.append(last_f)
-            x_history.append(np.copy(last_x))
-            mask.append(1)
+            last_f_try = problem(last_x)
+            if last_f_try < last_f:
+                last_f = last_f_try
+                global_state_history.append(deepcopy(problem.global_state))
+                f_history.append(last_f)
+                x_history.append(np.copy(last_x))
+                mask.append(1)
+            else:
+                mask.append(0)
+                f_history.append(last_f)
+                x_history.append((np.copy(x0)))
         except:
             mask.append(0)
             last_f = np.inf
@@ -348,35 +354,33 @@ class RandomOptimizer:
                     exception_list.append(e)
 
             if tk <= self.R:
-                return OptResult(x_history=np.fromiter(x_history, dtype=np.dtype((float, x0.shape))),
-                                 f_history=np.fromiter(map(float, f_history), dtype=np.float),
-                                 step_mask=np.fromiter(mask, dtype=int),
-                                 global_state_history=global_state_history,
-                                 status_code=0,
-                                 status_message='Оптимизация завершилась удачно по критерию останова',
-                                 error_list=exception_list
-                                 )
+                break
             else:
                 tk *= self.beta
                 bad_steps_cur = 1
 
-        if x0 == last_x:
-            return OptResult(x_history=np.fromiter(x_history, dtype=np.dtype((float, x0.shape))),
-                             f_history=np.fromiter(map(float, f_history), dtype=np.float),
-                             step_mask=np.fromiter(mask, dtype=int),
+        f_history = np.array(f_history)
+        x_history = np.array(x_history)
+        mask = np.array(mask).astype(bool)
+
+        if last_f == np.inf:
+            return OptResult(x_history=x_history,
+                             f_history=f_history,
+                             step_mask=mask,
                              global_state_history=global_state_history,
                              status_code=1,
+                             status_message='Оптимизация завершилась неудачно, т.к. не найдено ни одного оптимума',
+                             error_list=exception_list
+                             )
+        else:
+            return OptResult(x_history=x_history,
+                             f_history=f_history,
+                             step_mask=mask,
+                             global_state_history=global_state_history,
+                             status_code=0,
                              status_message='Оптимизация завершилась удачно по критерию останова',
                              error_list=exception_list
                              )
-        return OptResult(x_history=np.fromiter(x_history, dtype=np.dtype((float, x0.shape))),
-                         f_history=np.fromiter(map(float, f_history), dtype=np.float),
-                         step_mask=np.fromiter(mask, dtype=int),
-                         global_state_history=global_state_history,
-                         status_code=2,
-                         status_message='Оптимизация завершилась удачно по критерию останова',
-                         error_list=exception_list
-                         )
 
 class SRandomOptimizer:
 
@@ -546,15 +550,26 @@ class ConstrainedOptimizer:
             _f_history.extend(res.f_history)
             _global_state_history.extend(res.global_state_history)
             _mask.extend(res.step_mask)
+
+            if res.status_code == 1:
+                return OptResult(x_history=np.array(_x_history),
+                                 f_history=np.array(_f_history),
+                                 step_mask=np.array(_mask),
+                                 global_state_history=_global_state_history,
+                                 status_code=res.status_code,
+                                 status_message=res.status_message,
+                                 error_list=_error_list
+                                 ), constaint_status
+
             problem(res.x_history[-1])
             factor = problem.constraint_factor
             problem.callback()
 
             if abs(factor) < tol:
                 constaint_status = 1
-                return OptResult(x_history=np.fromiter(_x_history, dtype=np.dtype((float, _x_history[0].shape))),
-                                 f_history=np.fromiter(map(float, _f_history), dtype=np.float),
-                                 step_mask=np.fromiter(_mask, dtype=int),
+                return OptResult(x_history=np.array(_x_history),
+                                 f_history=np.array(_f_history),
+                                 step_mask=np.array(_mask),
                                  global_state_history=_global_state_history,
                                  status_code=res.status_code,
                                  status_message=res.status_message,
@@ -566,8 +581,8 @@ class ConstrainedOptimizer:
                          f_history=np.fromiter(map(float, _f_history), dtype=np.float),
                          step_mask=np.fromiter(_mask, dtype=int),
                          global_state_history=_global_state_history,
-                         status_code=2,
-                         status_message='Оптимизация завершилась удачно по критерию останова',
+                         status_code=res.status_code,
+                         status_message=res.status_message,
                          error_list=_error_list
                          ), constaint_status
 
